if (!BS.Clouds) {
  BS.Clouds = {};
}

BS.Clouds.Admin = {
  _runningInstancesCount : 0,

  setRunningInstancesCount : function(c) {
    this._runningInstancesCount = c;
  },

  getRunningInstancesCount : function() {
    return this._runningInstancesCount;
  },

  refresh : function() {
    $('cloudRefreshable').refresh();
  },

  registerRefresh : function() {
    Behaviour.addLoadEvent(function() {
      BS.Clouds.registerRefreshable(BS.Clouds.Admin.refresh.bind(BS.Clouds.Admin));
    });
  },

  createNewProfile : function(e) {
    return this._showDialog(e, "action=new");
  },

  updateProfile : function(e, profileId) {
    return this._showDialog(e, "action=edit&profileId=" + profileId);
  },

  _showDialog : function(e, params) {
    BS.Clouds.Admin.CreateProfileDialog.showProgress(e);

    BS.ajaxUpdater($("newProfileFormMainRefresh"),
        base_uri + "/clouds/admin/cloudAdminProfile.html?" + params,
    {
      method : "get",
      evalScripts : true,
      onComplete : function() {
        BS.Clouds.Admin.CreateProfileDialog.baseParams = function() {
          return params;
        };

        ((function() {
          var dialog = $(this.getContainer());
          var pos = BS.Util.computeCenter(dialog);
          this.showAt(pos[0], 100);
        }).bind(BS.Clouds.Admin.CreateProfileDialog))();
        $('profileName').focus();
      }
    });

    return false;
  },

  CreateProfileForm : OO.extend(BS.PluginPropertiesForm, {
    getDialog : function() {
      return BS.Clouds.Admin.CreateProfileDialog;
    },

    formElement : function() {
      return $('newProfileForm');
    },

    savingIndicator : function() {
      return $('newProfileProviderProgress');
    },

    getSelectedType : function() {
      var el = this.formElement().cloudType;
      var idx = el.selectedIndex;
      return el.options[idx].value;
    },

    saveForm : function() {
      var that = this;
      var url = this.formElement().getAttributeNode('action').value;
      BS.PasswordFormSaver.save(that, url, OO.extend(BS.ErrorsAwareListener, {
        onCompleteSave : function(form, responseXML, err) {
          var wereErrors = BS.XMLResponse.processErrors(responseXML, {}, that.propertiesErrorsHandler);

          BS.ErrorsAwareListener.onCompleteSave(form, responseXML, err);

          if (wereErrors) {
            BS.Util.reenableForm(that.formElement());
            return;
          }

          that.getDialog().close();
          BS.Clouds.Admin.refresh();
        }
      }));
    },

    beforeShow : function() {
      this.clearErrors();
      BS.Util.reenableForm(this.formElement());
    }
  }),

  enabledClouds : function(b, uri) {
    var btn = $(b);
    btn.disable();

    $('enable_integration_loader').show();
    BS.ajaxRequest(uri, {
      parameters : {
        action :  'enable'
      },
      method: 'post',
      onComplete : function() {
        BS.reload(true);
      }
    });
  },

  ConfirmDialog : OO.extend(BS.AbstractModalDialog, {
    aThis : null,

    getContainer : function() {
      return $('confirmShutdownDialog');
    },

    showDialog : function(caption, submitCaption, runningCount) {
      BS.Clouds.Admin.ConfirmDialog.aThis = this;
      $('confirmShutdown_action').innerHTML = caption;
      $('confirmShutdown_submit').value = submitCaption;
      $('confirmShutdown_instanceCount').innerHTML = runningCount;
      if (runningCount == 0) {
        $('terminateInstancesCheckbox').hide();
      }
      this.showCentered();
    },

    shouldKillInstances : function() {
      return $('terminateInstances').checked ? 'true' : 'false';
    },

    //Override this function to handle confirm
    onConfirm : function() {
    },

    submit : function() {
      $('confirmShutdown').disable();
      $('confirmShutdownDialog_loader').show();

      (this.aThis||this).onConfirm();

      return false;
    },

    postSubmit : function() {
      $('confirmShutdownDialog_loader').hide();
      $('confirmShutdown').enable();
      this.close();
    },

    processKillError : function(transport) {
      return BS.XMLResponse.processErrors(transport.responseXML, {
        onKillFailedError: function(elem) {
          alert(elem.firstChild.nodeValue + "\nAction is canceled due to the error");
        }
      });
    }
  }),

  CreateProfileDialog : OO.extend(BS.DialogWithProgress, {
    getForm : function() {
      return BS.Clouds.Admin.CreateProfileForm;
    },

    beforeShow : function() {
      this.getForm().beforeShow();
    },

    getContainer : function() {
      return $('newProfileFormDialog');
    },

    disableButton : function(ids, value) {
      if (value) {
        ids.each(function(item) { $(item).disabled = 'disabled';} );
      } else {
        ids.each(function(item) { $(item).disabled = '';} );
      }
      var that = this;
      return function() {
        that.disableButton(ids, !value);
      };
    },

    _baseParams : function() {
      return null;
    },

    refreshDialog : function() {
      var providerType = this.getForm().getSelectedType();
      if (providerType != '') {
        var enable = this.disableButton(["createButton", "cloudType"], true);
        var url = 'cloudType=' + providerType + "&" + this.baseParams();
        $('newProfilesContainer').refresh('newProviderSaving',  url, enable);
      } else {
        this.disableButton(["createButton"], true);
      }
    },

    submit : function() {
       var providerType = this.getForm().getSelectedType();
       if (providerType == '') {
         alert('Please select Cloud Type');
         return false;
       }
       this.getForm().saveForm();

       return false;
     }
  })
};

BS.Clouds.Admin.ConfirmDeleteProfileDialog = OO.extend(BS.Clouds.Admin.ConfirmDialog, {
  profileId : "",

  showDeleteProfileDialog : function(profileId, profileName, runningCount) {
    this.profileId = profileId;
    $("confirmShutdownTitle").innerHTML = "Remove profile confirmation";
    return this.showDialog('remove profile "' + profileName, 'Remove', runningCount);
  },

  onConfirm : function() {
    var that = this;
    BS.ajaxRequest(base_uri + "/clouds/admin/cloudAdminProfile.html", {
      parameters : {
        action : 'delete',
        profileId : that.profileId,
        killInstances : that.shouldKillInstances()
      },
      method : 'post',
      onComplete : function(transport) {
        var handled = that.processKillError(transport);
        if (!handled) {
          BS.Clouds.Admin.refresh();
        }
        that.postSubmit();
      }
    });
  }
});

BS.Clouds.Admin.ConfirmShutdownDialog = OO.extend(BS.Clouds.Admin.ConfirmDialog, {
  showConfirmDisableDialog: function() {
    var running = BS.Clouds.Admin.getRunningInstancesCount();
    $("confirmShutdownTitle").innerHTML = "Disable cloud integration";
    return this.showDialog('disable cloud integraion', 'Disable', running);
  },

  onConfirm : function() {
    var form = 'confirmShutdown';
    var url = $(form).getAttribute('action');

    var that = this;
    BS.ajaxRequest(url, {
      parameters : {
        action : 'disable',
        kill : that.shouldKillInstances()
      },
      method: 'post',
      onComplete : function(transport) {
        var handled = that.processKillError(transport);
        if (!handled) {
          BS.reload(true);
        }
        that.postSubmit();
      }
    });
  }
});


